//--------------------------------------------------------------
//
//  FUNCky.cc - FUNCky 6.0 Simplified dBASE Interface Classes
//
//  These simple Classes map the FUNCky COM Interfaces to 
//  Visual dBASE Classes of the same name. This allows 
//  familiar VDB coding constructs to be used. It doesn't
//  get any easier than this!
//
//  To use these Classes, FUNCky60.DLL must be registered
//  on the target system.
//
//  Copyright (c) 1988-2000, dLESKO Inc.                 
//  All rights reserved.
//
//---------------------------------------------------------------

// Main FUNCky Class with general purpose programming routines.
Class FUNCky of OleAutoClient("FUNCky")
endClass

// FUNCky Constants Class exposes all the constants used
// by all FUNCky routines as Properties.
Class FUNCkyConst of OleAutoClient("FUNCkyConst")
endClass

// FUNCky Date Class exposes Year, Month, Day, Week, Century
// and Decade properties with special behaviors that permit simple
// arithmetic expression to be used to accurately manipulate dates
// both forward and backward in time.
Class FUNCkyDate of OleAutoClient("FUNCkyDate")
endClass

// Probably the most uneeded Class there is for VDB users, but
// its in there, so might as well expose it.... Note: It uses
// Clipper style locking on the DBF and is not compatible with
// Visual dBASE DBF files. 
Class FUNCkyDBF of OleAutoClient("FUNCkyDBF")
endClass

// FUNCky Event Class. Events can be used to synchronize multiple
// threads and processes. Very handy when one application has
// to wait for another application to finish something before it
// can continue.
Class FUNCkyEvent of OleAutoClient("FUNCkyEvent")
endClass

// Need to write to the NT or Windows 2000 Event Log? FUNCky's
// EventLog Class can do it in two lines of code.
Class FUNCkyEventLog of OleAutoClient("FUNCkyEventLog")
endClass

// This low level file Class is a little more intuitive and
// simple to use than low level file functions. Its control
// over end of line terminators and line lengths make it easier
// to use with delimited text files.
Class FUNCkyFile of OleAutoClient("FUNCkyFile")
endClass

// A Finder Class - this one browses for Domains or Workgroups.
Class FUNCkyFindDomain of OleAutoClient("FUNCkyFindDomain")
endClass

// A Finder Class - this one browses for configured ODBC Data Sources.
Class FUNCkyFindDSN of OleAutoClient("FUNCkyFindDSN")
endClass

// A Finder Class - this one browses for available OLEDB Providers.
Class FUNCkyFindOleDB of OleAutoClient("FUNCkyFindOleDB")
endClass


// A Finder Class - this one browses for files and directories.
// Its filtering and multiple file specifications blow all
// other file locating classes away for ease of use.
Class FUNCkyFindFile of OleAutoClient("FUNCkyFindFile")
endClass

// A Finder Class - this one browses for all logged in users   
// on an NT or Windows 2000 system. This is useful to track down
// the location of a particular user.
Class FUNCkyFindLogon of OleAutoClient("FUNCkyFindLogon")
endClass

// A Finder Class - this one browses for open files on an   
// NT or Windows 2000 system. It can also report who has the
// file opened, for how long, and, it can force files to be
// closed (useful for forced system shutdown and maintenance).
Class FUNCkyFindOpenFile of OleAutoClient("FUNCkyFindOpenFile")
endClass

// A Finder Class - this one browses for all printers installed
// on the current system. Useful to allow users to select
// a Printer to print to.
Class FUNCkyFindPrinter of OleAutoClient("FUNCkyFindPrinter")
endClass

// A Finder Class - this one browses for all available network 
// servers in the current or a specified Domain.            
Class FUNCkyFindServer of OleAutoClient("FUNCkyFindServer")
endClass

// A Finder Class - this one browses for NT or Windows 2000  
// services. Good to allow users to pick a service that can
// then be opened and manipulated with the FUNCkyService Class.
Class FUNCkyFindService of OleAutoClient("FUNCkyFindService")
endClass

// A Finder Class - this one browses for User Sessions on the
// current system or a remote system. Useful to see who is
// connected to what system.
Class FUNCkyFindSession of OleAutoClient("FUNCkyFindSession")
endClass

// A Finder Class - this one browses for available Shares
// on the current or a specified system.            
Class FUNCkyFindShare of OleAutoClient("FUNCkyFindShare")
endClass

// FUNCky FTP Class that can Put and Get files from an FTP Server.
Class FUNCkyFTP of OleAutoClient("FUNCkyFTP")
endClass

// FUNCky Mutex Class. Mutexes allow synchronized access to global
// resources. The global resource can be a file, directory,
// FUNCkyView object, COMM port, you name it. If only one
// process at a time should have access to something, a FUNCky
// Mutex is the way to protect it. A Mutex can also
// be used to detect if another instance of your application
// is already running.
Class FUNCkyMutex of OleAutoClient("FUNCkyMutex")
endClass

// FUNCky NetStat Class. This gives a snapshot of all the current TCP
// and UDP sessions on the current system. Very useful for
// monitoring network stats. This information is the same as
// that which can be obtained by running NetStat -a in a
// command window.
Class FUNCkyNetStat of OleAutoClient("FUNCkyNetStat")
endClass

// FUNCky Registry Class. This has got to be the easiest way to
// manipulate the registry. Just open a key, and treat the
// values as though they were properties! It can even preserve
// the data type in the registry.
Class FUNCkyRegistry of OleAutoClient("FUNCkyRegistry")
endClass

// This Text File Class makes working with delimited
// rows of data a snap. Just create some field definitions
// and open a file and instantly the character offsets and
// positions you define turn into properties of the file
// object. No more fussing with messy calls to SubStr()
// and Stuff() to try and parse data from delimited text.
Class FUNCkySDF of OleAutoClient("FUNCkySDF")
endClass

// FUNCky Semaphore Class. Semaphores allow synchronized access to global
// resources. The global resource can be a file, directory,
// FUNCkyView object, COMM port, you name it. Semaphores use
// a resource count to limit how many processes can access
// a global resource at one time. Very useful for COMM ports.
// If, say, there are only two COMM ports. A Semaphore can limit
// a third process from connecting to a COMM port if two processes
// are already using them.
Class FUNCkySemaphore of OleAutoClient("FUNCkySemaphore")
endClass

// FUNCky Service Class - Need to stop and start NT or Windows 2000
// Services? Then this class is for you.
Class FUNCkyService of OleAutoClient("FUNCkyService")
endClass

// FUNCky SMTP Class that can send email to a mail server.
Class FUNCkySMTP of OleAutoClient("FUNCkySMTP")
endClass

// FUNCky TCP Class that can send and receive data from
// a TCP server.
Class FUNCkyTCP of OleAutoClient("FUNCkyTCP")
endClass

// FUNCky Timer Class. VDB does not yet subscribe to
// COM connection points so the Timer Events cannot
// be seen... but, maybe one day. For now, this Class
// can be seen but not heard... (see VB demo if you dare)
Class FUNCkyTCP of OleAutoClient("FUNCkyTCP")
endClass

// FUNCky Trace Route Class that can map a route to
// a specific server just like the TracerT command line
// program.
Class FUNCkyTraceRoute of OleAutoClient("FUNCkyTraceRoute")
endClass

// FUNCky Trigger Class. If VDB could expose the events
// of this Class, you could monitor a specific folder for
// changes to its files and directories. However, VDB does
// not yet subscribe to COM connection points so the
// Trigger Events cannot be seen... but, maybe one day.
// For now, this Class can be seen but not heard... (see
// VB demo if you dare)
Class FUNCkyTrigger of OleAutoClient("FUNCkyTrigger")
endClass

// FUNCky Version Info Class. Undoubtably, the easiest
// way to extract Version Information from Exe, OCX and DLL
// files. All Version info is exposed as properties.
Class FUNCkyVersion of OleAutoClient("FUNCkyVersion")
endClass

// FUNCky View Class. Need to share information with other
// running processes? Need to know how many times the
// current process has been run? Want to re-use current
// program settings in a second instance of your application?
// Then a View is for you. A FUNCky View object allows data to
// be publically visible to all interested processes. The data
// is mapped by reading and writing to properties of the
// view object - and, the data types are preserved! There is no
// easier way on the planet to share data with other processes.
Class FUNCkyView of OleAutoClient("FUNCkyView")
endClass

// Nifty function that simulates VB's CreateObject() function call.
// This helps the FUNCky example code to more resemble the actual code.
FUNCTION CreateObject(Name)
return New OleAutoClient(Name)

               /// End of FUNCky 6.0 Classes ///
                    /// See you in 7.0! ///
